<?php

namespace Drupal\slidesjs_slider\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\file\Entity\File;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Provides a 'SlidejsSliderBlock' block.
 *
 * @Block(
 *  id = "slidesjs_slider_block",
 *  admin_label = @Translation("SlidesJS Slider"),
 *  category = @Translation("Custom")
 * )
 */
class SlidejsSliderBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new SlidejsSliderBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, FileSystemInterface $file_system, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fileSystem = $file_system;
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'slides' => [],
      'settings' => [
        'speed' => 3000,
        'auto_play' => TRUE,
        'show_navigation' => TRUE,
        'show_pagination' => TRUE,
        'slider_size' => 50, // Default size in percentage
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    // Global settings.
    $form['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Slider Settings'),
      '#open' => TRUE,
    ];

    $form['settings']['slider_size'] = [
      '#type' => 'range',
      '#title' => $this->t('Slider Size'),
      '#description' => $this->t('Adjust the overall size of the slider (affects both height and width).'),
      '#min' => 20,
      '#max' => 100,
      '#step' => 5,
      '#default_value' => $config['settings']['slider_size'] ?? 50,
      '#attributes' => [
        'oninput' => 'this.nextElementSibling.value = this.value + "%"; this.nextElementSibling.style.display = "inline-block";',
      ],
      '#field_suffix' => [
        '#type' => 'text',
        '#value' => ($config['settings']['slider_size'] ?? 50) . '%',
        '#attributes' => [
          'class' => ['slider-size-display'],
          'style' => ['display: inline-block;'],
        ],
      ],
    ];

    $form['settings']['speed'] = [
      '#type' => 'number',
      '#title' => $this->t('Speed (ms)'),
      '#description' => $this->t('Time between slide transitions in milliseconds.'),
      '#min' => 1000,
      '#max' => 10000,
      '#step' => 100,
      '#default_value' => $config['settings']['speed'] ?? 3000,
    ];

    $form['settings']['auto_play'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Auto Play'),
      '#description' => $this->t('Automatically transition between slides.'),
      '#default_value' => $config['settings']['auto_play'] ?? TRUE,
    ];

    $form['settings']['show_navigation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Navigation Arrows'),
      '#description' => $this->t('Show previous/next navigation arrows.'),
      '#default_value' => $config['settings']['show_navigation'] ?? TRUE,
    ];

    $form['settings']['show_pagination'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Pagination'),
      '#description' => $this->t('Show pagination dots below the slider.'),
      '#default_value' => $config['settings']['show_pagination'] ?? TRUE,
    ];

    // Slides configuration.
    $form['slides'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Slides'),
      '#tree' => TRUE,
      '#prefix' => '<div id="slidesjs-slides-wrapper">',
      '#suffix' => '</div>',
    ];

    // Get the number of slides.
    $slides_count = $form_state->get('slides_count');
    if (NULL === $slides_count) {
      $slides_count = !empty($config['slides']) ? count($config['slides']) : 1;
      if ($slides_count < 1) {
        $slides_count = 1;
      }
      $form_state->set('slides_count', $slides_count);
    }

    // Create form elements for each slide.
    for ($i = 0; $i < $slides_count; $i++) {
      $form['slides'][$i] = [
        '#type' => 'details',
        '#title' => $this->t('Slide @num', ['@num' => $i + 1]),
        '#open' => TRUE,
      ];

      $form['slides'][$i]['image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Image'),
        '#description' => $this->t('Upload a slider image. Recommended size: 1920x800 pixels.'),
        '#upload_location' => 'public://slidesjs_slider/',
        '#upload_validators' => [
          'file_validate_extensions' => ['png gif jpg jpeg'],
          'file_validate_is_image' => [],
        ],
        '#default_value' => $config['slides'][$i]['image'] ?? NULL,
        '#required' => TRUE,
      ];

      $form['slides'][$i]['title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#default_value' => $config['slides'][$i]['title'] ?? '',
        '#maxlength' => 255,
      ];

      $form['slides'][$i]['description'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Description'),
        '#default_value' => $config['slides'][$i]['description'] ?? '',
        '#rows' => 3,
      ];

      $form['slides'][$i]['link'] = [
        '#type' => 'url',
        '#title' => $this->t('Link URL'),
        '#default_value' => $config['slides'][$i]['link'] ?? '',
        '#description' => $this->t('Optional: URL to link this slide to.'),
      ];
    }

    // Add button to add more slides.
    $form['add_slide'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Another Slide'),
      '#submit' => [[$this, 'addMoreSlides']],
      '#ajax' => [
        'callback' => [$this, 'updateSlidesCallback'],
        'wrapper' => 'slidesjs-slides-wrapper',
      ],
    ];

    // Only show "Remove" button if there's more than one slide.
    if ($slides_count > 1) {
      $form['remove_slide'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove Last Slide'),
        '#submit' => [[$this, 'removeSlide']],
        '#ajax' => [
          'callback' => [$this, 'updateSlidesCallback'],
          'wrapper' => 'slidesjs-slides-wrapper',
        ],
      ];
    }

    return $form;
  }

  /**
   * Callback for adding more slides.
   */
  public function addMoreSlides(array &$form, FormStateInterface $form_state) {
    $slides_count = $form_state->get('slides_count');
    $form_state->set('slides_count', $slides_count + 1);
    $form_state->setRebuild();
  }

  /**
   * Callback for removing slides.
   */
  public function removeSlide(array &$form, FormStateInterface $form_state) {
    $slides_count = $form_state->get('slides_count');
    if ($slides_count > 1) {
      $form_state->set('slides_count', $slides_count - 1);
    }
    $form_state->setRebuild();
  }

  /**
   * Ajax callback for slide form updates.
   */
  public function updateSlidesCallback(array &$form, FormStateInterface $form_state) {
    return $form['slides'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['settings'] = $form_state->getValue('settings');
    $slides = $form_state->getValue('slides');

    // Process each slide and make file usage permanent.
    foreach ($slides as $i => $slide) {
      if (!empty($slide['image'])) {
        // Make uploaded files permanent.
        $image_file = File::load($slide['image'][0]);
        if ($image_file) {
          $image_file->setPermanent();
          $image_file->save();
        }
      }
    }

    $this->configuration['slides'] = $slides;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $slides = $config['slides'];
    $processed_slides = [];

    foreach ($slides as $i => $slide) {
      if (!empty($slide['image'][0])) {
        $file = File::load($slide['image'][0]);
        if ($file) {
          $processed_slides[$i] = [
            'image_uri' => $file->getFileUri(),
            'title' => $slide['title'] ?? '',
            'description' => $slide['description'] ?? '',
            'link' => $slide['link'] ?? '',
          ];
        }
      }
    }

    return [
      '#theme' => 'slidesjs_slider_block',
      '#slides' => $processed_slides,
      '#settings' => $config['settings'],
      '#attached' => [
        'library' => [
          'slidesjs_slider/slidesjs_slider.slider',
        ],
      ],
    ];
  }

} 