<?php

namespace Drupal\slidesjs_slider;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\block\Entity\Block;
use Drupal\file\Entity\File;
use Drupal\Core\File\FileRepositoryInterface;

/**
 * Service for managing SlidesJS Slider operations.
 */
class SliderManager {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The file repository service.
   *
   * @var \Drupal\Core\File\FileRepositoryInterface
   */
  protected $fileRepository;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new SliderManager.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\File\FileRepositoryInterface $file_repository
   *   The file repository service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    $file_repository,
    ConfigFactoryInterface $config_factory,
    MessengerInterface $messenger
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->fileRepository = $file_repository;
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
  }

  /**
   * Creates a demo slider block with the provided sample images.
   *
   * @param array $image_files
   *   Array of file IDs to use for the demo slider.
   *
   * @return \Drupal\block\Entity\Block|null
   *   The created block or NULL if creation failed.
   */
  public function createDemoSlider(array $image_files) {
    if (empty($image_files)) {
      return NULL;
    }

    // Get the default theme.
    $theme_name = $this->configFactory->get('system.theme')->get('default');
    
    // Set up slide configurations.
    $slides = [
      [
        'image' => [$image_files[0]],
        'title' => t('Welcome to SlidesJS Slider'),
        'description' => t('A modern, responsive slider for Drupal 10.'),
        'link' => '',
      ],
    ];

    // Add additional slides if we have more images.
    if (isset($image_files[1])) {
      $slides[] = [
        'image' => [$image_files[1]],
        'title' => t('Fully Customizable'),
        'description' => t('Upload your own images and customize the slider settings.'),
        'link' => '',
      ];
    }

    if (isset($image_files[2])) {
      $slides[] = [
        'image' => [$image_files[2]],
        'title' => t('Mobile Friendly'),
        'description' => t('Works great on all devices with touch support.'),
        'link' => '',
      ];
    }
    
    // Block settings.
    $settings = [
      'speed' => 4000,
      'auto_play' => TRUE,
      'show_navigation' => TRUE,
      'show_pagination' => TRUE,
    ];
    
    // Check if hero region exists in the theme.
    $theme_regions = system_region_list($theme_name);
    $region = isset($theme_regions['hero']) ? 'hero' : 'content';

    // Create block entity.
    $values = [
      'id' => 'slidesjs_slider_demo',
      'plugin' => 'slidesjs_slider_block',
      'region' => $region,
      'settings' => [
        'id' => 'slidesjs_slider_block',
        'label' => 'SlidesJS Slider Demo',
        'provider' => 'slidesjs_slider',
        'label_display' => '0',
        'slides' => $slides,
        'settings' => $settings,
      ],
      'theme' => $theme_name,
      'visibility' => [],
      'weight' => -100,
    ];

    // Save the block.
    $block = Block::create($values);
    $block->save();
    
    return $block;
  }

  /**
   * Process an image file for the slider.
   *
   * @param string $source_path
   *   The source path of the image.
   * @param string $destination_directory
   *   The destination directory for the processed image.
   * @param string $filename
   *   The filename to use for the processed image.
   *
   * @return \Drupal\file\FileInterface|null
   *   The processed file entity or NULL if processing failed.
   */
  public function processImage($source_path, $destination_directory, $filename) {
    if (!file_exists($source_path)) {
      return NULL;
    }

    // Ensure the destination directory exists.
    $this->fileSystem->prepareDirectory($destination_directory, FileSystemInterface::CREATE_DIRECTORY);
    
    // Destination URI
    $uri = $destination_directory . $filename;
    
    // Use file_system to copy the file
    $this->fileSystem->copy($source_path, $uri, FileSystemInterface::EXISTS_REPLACE);
    
    // Create a managed file entity
    $file = File::create([
      'uri' => $uri,
      'status' => 1,
    ]);
    
    $file->save();
    return $file;
  }

  /**
   * Cleans up files and blocks created by this module.
   */
  public function cleanUp() {
    // Delete the demo block if it exists.
    $block_storage = $this->entityTypeManager->getStorage('block');
    $block = $block_storage->load('slidesjs_slider_demo');
    if ($block) {
      $block->delete();
    }

    // Clean up uploaded files.
    $file_storage = $this->entityTypeManager->getStorage('file');
    
    // Find files in our designated directory.
    $files = $file_storage->loadByProperties(['uri' => 'public://slidesjs_slider/%']);
    foreach ($files as $file) {
      $file->delete();
    }

    // Remove the directory.
    $this->fileSystem->deleteRecursive('public://slidesjs_slider');
  }

} 